//*****************************************
//Pica um LED (PORTD) com interrupo interna TMR0 com tempo de 200 ms

//*****************************************
//1 Conf. Temporizao do timer
//2 Conf. "chave geral" das interrupes
//3 Conf. a interrupo particular
//4 Escrever vetores de interrupo (HIGH OU LOW)
//5 Subrotina de interrupo
//******************************************

//Prottipo de Funo
void ConfigMCU();
void ConfigTIMER();
void ConfigInterrupt_Global();
void ConfigInterrupt_TIMER0();


//Rotina Especial de Interrupo gerada pelo Assistante do mikroC
// menu Tools > Interrupt Assistant
void Interrupt_HIGH() iv 0x0008 ics ICS_AUTO {   //Alta prioridade de interrupo

   if(INTCON.TMR0IF == 1)    //Foi o TIMER0 que gerou a interrupo ?
    {
      PORTD.RD0 = ~LATD.RD0; //PISCA O LED

      TMR0H = 0XCF;          //Recarregar o TIMER para 200ms
      TMR0L = 0X2C;
      INTCON.TMR0IF = 0;    //No esquecer de zerar a Flag, pois  responsvel por
                            //acionar a interrupo
    }

}    // Fim da subrotina de interrupo


void ConfigMCU()
{
#ifdef P18F45K22
 ANSELD = 0;
 ANSELB = 0;
#else
 ADCON1 |= 0X0F;
#endif

 TRISD = 0;       //Configurar os pinos de controle dos LEDs
 PORTD = 0;
}

void ConfigTIMER()
{
//*******************TIMER0 PARA 200ms*********************************
//CicloMaquina = Fosc/4 -> 8MHz/4 = 2MHz -> Periodo = 1/F -> 1/2MHz = 0.5us

//Time_overflow = CicloMaq. * Prescaler * (Modo_8_16bits - valor Inicial)
//200.000us    = 0.5us * 32 * (65536 -  X)
//X = 65536 - 12500 = 53036  -> converter valor para HEX: CF2C

//******************************************************************************
  T0CON = 0B00000100;  //TIMER_OFF, MOD_16BITS, TIMER, PRES_1:32
  TMR0H = 0XCF;   // carga do valor inicial
  TMR0L = 0X2C;

  INTCON.TMR0IF = 0;  //zera o Flag
  T0CON.TMR0ON = 1;   //Liga o TIMER0
}

void ConfigInterrupt_Global()
{
  INTCON.GIEH = 1;   //Chave Geral, Habilita o vetor de alta prioridade de interrupo
  INTCON.GIEL = 1;   //Habilita o vetor de baixa prioridade (obs: no  chave geral)
  RCON.IPEN = 1; //0 -> (default) herana da famlia PIC16F, onde existe apenas
                            //um nvel de prioridade de interrupo
                     //1 -> dois (2) nveis de prioridade de interrupo encontrado no PIC18
}

void ConfigInterrupt_TIMER0()
{

// IF, IP, IE + perifrico
//TMR0 + IF = TMR0IF
// TMR0 + IP = TMR0IP
// ' ' = TMR0IE
// Registradores que contem os bits TMR0IF e TMR0IE = INTCON; TMR0IP = INTCON2

  INTCON.TMR0IF = 0;   //Flag de estouro do TIMER0. Deve iniciar em 0
  INTCON2.TMR0IP = 1;  //Prioridade de interrupo do TIMER0. Conf. em Alta prioridade
  INTCON.TMR0IE = 1;   //Habilita a interrupo individual do TIMER0
}


void main() {
   ConfigMCU();           //pinos do MCU config. para acionar os LEDS
   ConfigTIMER();         // tempo config. e TIMER ligado
   ConfigInterrupt_Global(); //Configurao global das interrupes
   ConfigInterrupt_TIMER0(); // interrupo individual do TIMER0  habilitada

  //while(1) {
 // }
}